module Main where

import System
import System.Console.GetOpt
import List

import AbsSyn
import Lex
import Parse
import Interp

data Flag = Verbose
          | Help 
          | Traces String
          | Output String
            deriving (Eq, Show)

optDescrs :: [OptDescr Flag]
optDescrs =
  [ Option ['h','?'] ["help"]   (NoArg Help)     "show usage information"
  , Option ['v']     ["verbose"] (NoArg Verbose) "be verbose"
  , Option ['t']     ["traces"] (ReqArg Traces "Traces") "list of tracing flags"
  , Option ['o']     ["output"] (ReqArg Output "FILE") "output file"
  ]

usage :: [String] -> IO a 
usage errs = ioError (userError (concat errs ++ usageInfo header optDescrs)) 
    where header = "Usage: main [options] <file>" 

data Options = Options {
    verbose :: Bool,
    traces :: [String]
  }

emptyOpts :: Options
emptyOpts = Options False []


main =	do
  cmdLine  <- getArgs
  let (os, ns, errs) = getOpt Permute optDescrs cmdLine
  if (length ns)<1
    then usage [""]
    else return ()
  let (prgFile:args) = ns
  putStrLn $ "Reading program from file: " ++ prgFile
  input <- readFile prgFile
  putStrLn $ "Passing these args to main fct: " ++ (show args)
  -- input <- getContents
  -- print (alexScanTokens input)
  let ast = parse (alexScanTokens input)
  --putStrLn ("AST:\n" ++ show ast)
  --let res = Interp.runIntp (Interp.initState ast) (Interp.eval 
  let (Just main_ast) = find (\ (AbsSyn.Fct name _ _) -> name == "Lmain") ast
  let (msgs,res) = Interp.execCall ast main_ast (map read args)
  putStrLn ("Messages:\n" ++ unlines msgs)
  putStrLn ("Result:\n" ++ (show res))