{
module Parse  where

-- import Alex

import Lex
import Tokens
import AbsSyn
import Debug.Trace(trace)
}

%tokentype { (Token AlexPosn) }

%name parse

%token
  	'{'             { LBrace _ }
  	'}'		{ RBrace _ }
  	'['		{ LBrack _ }
  	']'		{ RBrack _ }
  	'('		{ LPar _ }
  	')'		{ RPar _ }
   	'<-' 		{ Ass _ } 
  	'?'             { Ife _ }
  	':' 		{ Colon _ }
  	',' 		{ Comma _ }
  	';' 		{ Semicol _ }
  	'&&'		{ And _ } 
  	'||'		{ Or _ } 
   	'=' 		{ Def _ } 
   	'==' 		{ Eq _ } 
   	'!=' 		{ Ne _ } 
   	'<'		{ Lt _ } 
   	'<='		{ Le _ } 
   	'>'		{ Gt _ } 
   	'>='		{ Ge _ } 
  	'+' 		{ Plus _ }
  	'-'		{ Minus _ } 
  	'*' 		{ Times _ }
  	'/' 		{ Div _ }
  	'&' 		{ Amp _ }
  	EXP             { Exp _ }
  	MEM             { Mem _ }
  	SEQ             { Seq _ }
  	CALL             { Call _ }
  	FCT             { Tokens.Fct _ }
  	REG             { Reg _ }
  	LOC             { Loc _ }
  	ADR             { Adr _ }
  	JUMP            { Jump _ }
  	CJUMP           { CJump _ }
        integer         { Const $$ _ }
        label           { Label $$ _ }
        temp            { Register $$ _ }
        identifier      { Id $$ _ }


%left '<-'
%left '||'
%left '&&'
%left '<' '<=' '>' '>=' '==' '!='
%left '+'  '-'
%left '*'  '/'
%left '&' 

%%

FctList :: { [AbsSyn.Fct] }
FctList: {- empty -} {[]}
| Fct FctList { $1:$2 }

Fct :: { AbsSyn.Fct }
Fct: FCT label '(' ParamList ')' '=' '{' StmList '}' { AbsSyn.Fct $2 $4 $8 }

StmList :: { [AbsSyn.Stm] }
StmList: {- empty -}    { [] }
  | Stm StmList0       { $1:$2 }

StmList0 :: { [AbsSyn.Stm] }
StmList0: {- empty -}   { [] }
  | Stm StmList0   { $1:$2 }

Stm :: { AbsSyn.Stm }
Stm:  Exp '<-' Exp                { AbsSyn.MOVE $1 $3 }
|  Exp                        { AbsSyn.EXP $1 }
|  JUMP Exp '[' LabelList ']'     { AbsSyn.JUMP $2 $4 }
|  CJUMP '(' Exp RelOp Exp ')' '?'  label ':'  label { AbsSyn.CJUMP $4 $3 $5 $8 $10 }
|  SEQ '(' Stm ',' Stm ')'     { AbsSyn.SEQ $3 $5 }
|  label ':'                   { AbsSyn.LABEL $1 }

Exp :: { AbsSyn.Exp }
Exp: label          { AbsSyn.NAME $1 }
| temp              { AbsSyn.TEMP $1 }
| integer           { AbsSyn.CONST $1 }
| Exp '+' Exp { AbsSyn.BINOP AbsSyn.PLUS $1 $3 }
| Exp '-' Exp { AbsSyn.BINOP AbsSyn.MINUS $1 $3 }
| Exp '*' Exp { AbsSyn.BINOP AbsSyn.MUL $1 $3 }
| MEM  '(' Exp ')'           { AbsSyn.MEM $3 }
| '(' Exp ')'       { $2 }
| CALL Exp '(' ExpList ')'      { AbsSyn.CALL $2 $4 }
| '{' Stm ',' Exp '}' { AbsSyn.ESEQ $2 $4 }

ExpList :: { [AbsSyn.Exp] }
ExpList: {- empty -}    { [] }
  |  Exp ExpList0        { $1:$2 }

ExpList0 :: { [AbsSyn.Exp] }
ExpList0: {- empty -}    { [] }
  |  ',' Exp ExpList0        { $2:$3 }

LabelList :: { [AbsSyn.Label] }
LabelList: {- empty -}    { [] }
  | label LabelList0       { $1:$2 }

LabelList0 :: { [AbsSyn.Label] }
LabelList0: {- empty -}   { [] }
  | ',' label LabelList0   { $2:$3 }

Param: LOC integer   { AbsSyn.LOC $2 }
  | REG temp         { AbsSyn.REG $2 }
  | ADR integer      { AbsSyn.ADR $2 }

ParamList :: { [AbsSyn.Param] }
ParamList: {- empty -}    { [] }
  | Param ParamList0       { $1:$2 }

ParamList0 :: { [AbsSyn.Param] }
ParamList0: {- empty -}   { [] }
  | ',' Param ParamList0   { $2:$3 }

BinOpExp :: { AbsSyn.Exp }
BinOpExp: Exp BinOp Exp     { AbsSyn.BINOP $2 $1 $3 }

BinOp :: { AbsSyn.BinOp }
BinOp: '+'   { AbsSyn.PLUS }
     | '-'   { AbsSyn.MINUS }
     | '*'   { AbsSyn.MUL }
     | '/'   { AbsSyn.DIV }
     | '&&'  { AbsSyn.AND }
     | '||'  { AbsSyn.OR }

RelOp :: { AbsSyn.RelOp }
RelOp: '=='   { AbsSyn.EQ }
     | '!='   { AbsSyn.NE }
     | '<'   { AbsSyn.LT }
     | '>'   { AbsSyn.GT }
     | '<='   { AbsSyn.LE }
     | '>='   { AbsSyn.GE }


{
fst3 (x,_,_) = x
snd3 (_,y,_) = y
thd3 (_,_,z) = z

-- toOperation :: (Show a) => Token a -> Operation */
-- toOperation (OpC x _) = toEnum (fromEnum x) */
-- toOperation x = error $ "illegal token " ++ (show x) ++ " found instead of an OpCode" */

--happyError :: [Token AlexPosn] -> a
--happyError pos = error $ "Parse error at" ++ (show pos)

happyError :: [Token AlexPosn] -> a
happyError tks = error ("Parse error at " ++ lcn ++ "\n")
	where
	lcn = 	case tks of
		  [] -> "end of file"
		  (tk:_) -> "line " ++ show l ++ ", column " ++ show c ++ " (token " ++ filterPn (show tk) ++ ")"
			where AlexPn _ l c = token_pos tk
}
